@@ -8,13 +8,14 @@ from collections import defaultdict
 import csv
 
 def get_mapping(ec_dir='/home/users/assafp/Database/EcoCyc/current/data',
-                utr_len=100, inc_rep_pos=False):
+                utr_len=100, inc_rep_pos=False, linear_chromosome_list=None):
     """
     Get the mapping directly
     Arguments:
     - `ec_dir`: EcoCyc dir
     - `utr_len`: For estimating UTRs
     - `inc_rep_pos`: Include REP positions in map?
+    - `linear_chromosome_list`: list of linear chromosome/plasmid names.
     """
     tu_promoters = read_promoters_data(ec_dir)
     tu_terminators = read_terminators_data(ec_dir)
@@ -34,7 +35,7 @@ def get_mapping(ec_dir='/home/users/assafp/Database/EcoCyc/current/data',
         fsa_lens[fn] = len(fs)
     maps = position_to_gene(
         uid_pos, uid_tudata, tu_promoters, tu_terminators, tu_genes,
-        sRNAs_list+other_RNAs_list, fsa_lens, rep_pos=rep_pos, utr_len=utr_len)
+        sRNAs_list+other_RNAs_list, fsa_lens, rep_pos=rep_pos, utr_len=utr_len, lin_list=linear_chromosome_list)
     return maps, uid_names
 
 def read_REP_table(rep_file):
@@ -68,7 +69,7 @@ def read_fsas(ec_dir='/home/users/assafp/Database/EcoCyc/current/data'):
     from Bio import SeqIO
     fsa_seqs = {}
     for fsa_name in glob.glob("%s/*-*.fsa"%(ec_dir)):
-        fsa_seqs[fsa_name.rsplit('.',1)[0].split('-',1)[1]] =\
+        fsa_seqs[fsa_name.rsplit('/',1)[1].rsplit('.',1)[0].split('-',1)[1]] =\
                      SeqIO.read(open(fsa_name), 'fasta').seq
     return fsa_seqs
 
@@ -82,12 +83,32 @@ def generate_transcripts_file(
     - `utr_len`: Default UTR len
     - `ec_dir`: EcoCyc data dir
     - `chr_dict`: map chromosome name in EcoCyc to another name (e.g. chr)
+    Raises:
+        ValueError: if a gene chromosome is not in the bc-dir
     """
     tu_promoters = read_promoters_data(ec_dir)
     tu_terminators = read_terminators_data(ec_dir)
     uid_pos, uid_names, uid_tudata, sRNAs_list, other_RNAs_list, rRNAs =\
         read_genes_data(ec_dir)
     tu_genes = defaultdict(list)
+
+    fsa_seqs = read_fsas(ec_dir)
+    fsa_lens = {}
+    for fn, fs in fsa_seqs.items():
+        fsa_lens[fn] = len(fs)
+
+    # Rename chromosomes according to chr_dict
+    if chr_dict:
+        chr_lens = {}
+
+        for fn, length in fsa_lens.items():
+            if fn in chr_dict:
+                chr_lens[chr_dict[fn]] = length
+            else:
+                chr_lens[fn] = length
+
+        fsa_lens = chr_lens
+
     for gene, tus in uid_tudata.items():
         if not tus:
             tu_genes[gene] = [gene]
@@ -97,11 +118,17 @@ def generate_transcripts_file(
     for tu in tu_genes:
         tu_str = uid_pos[tu_genes[tu][0]][3]
         tu_chrn = uid_pos[tu_genes[tu][0]][0]
+
         if chr_dict:
             try:
                 tu_chrn = chr_dict[tu_chrn]
             except KeyError:
                 pass
+
+        # Report genes that do not match any chromosome. an exception is caught in the generate_transcripts_gff main()
+        if tu_chrn not in fsa_lens:
+            raise ValueError('the chromosome: %s cuold not be found in the bc-dir given.' % tu_chrn)
+
         tu_boundaries[tu] = [0, 0, tu_str, tu_chrn]
         if tu in tu_promoters:
             if tu_str == '+':
@@ -112,10 +139,10 @@ def generate_transcripts_file(
             # Get first gene
             if tu_str == '+':
                 first_pos = min([uid_pos[gene][1] for gene in tu_genes[tu]])
-                tu_boundaries[tu][0] = first_pos-utr_len
+                tu_boundaries[tu][0] = max(first_pos-utr_len, 0)
             else:
                 last_pos = max([uid_pos[gene][2] for gene in tu_genes[tu]])
-                tu_boundaries[tu][1] = last_pos+utr_len
+                tu_boundaries[tu][1] = min(last_pos+utr_len, fsa_lens[tu_chrn])
         if tu in tu_terminators:
             if tu_str == '+':
                 tu_boundaries[tu][1] = max(tu_terminators[tu])+1
@@ -125,10 +152,10 @@ def generate_transcripts_file(
             # Get first gene
             if tu_str == '-':
                 first_pos = min([uid_pos[gene][1] for gene in tu_genes[tu]])
-                tu_boundaries[tu][0] = first_pos-utr_len
+                tu_boundaries[tu][0] = max(first_pos-utr_len, 0)
             else:
                 last_pos = max([uid_pos[gene][2] for gene in tu_genes[tu]])
-                tu_boundaries[tu][1] = last_pos+utr_len
+                tu_boundaries[tu][1] = min(last_pos+utr_len, fsa_lens[tu_chrn])
     # Now write all the tus to a gff file
     for tu, tub in tu_boundaries.items():
         outfile.write(
@@ -183,7 +210,7 @@ def read_promoters_data(
             if line.startswith('COMPONENT-OF'):
                 tutemp = line.strip().split()[-1]
                 if tutemp.startswith('TU'):
-                    tuname = tutemp
+                    tuname = tutemp.replace('TU', 'IGT')
             if line.startswith('//'):
                 if tuname and p1pos:
                     tu_promoters[tuname] = p1pos
@@ -218,7 +245,7 @@ def read_terminators_data(
             if line.startswith('COMPONENT-OF'):
                 tutemp = line.strip().split()[-1]
                 if tutemp.startswith('TU'):
-                    tuname = tutemp
+                    tuname = tutemp.replace('TU', 'IGT')
             if line.startswith('//'):
                 if tuname and left_pos and right_pos:
                     tu_promoters[tuname] = (left_pos, right_pos)
@@ -234,7 +261,7 @@ def read_genes_data(
     gfile = 'genes.dat', sRNAs_types=('BC-2.2','BC-2.2.7'),
     other_RNAs_types=('BC-2.2.5', 'BC-2.2.6', 'BC-3.1.3.6'),
     exclude_set=('EG10438',),
-    rRNA_prod='RRNA'):
+    rRNA_prod=['RRNA']):
     """
     Read the genes.dat from the ec directory 
     Arguments:
@@ -247,7 +274,7 @@ def read_genes_data(
     - `uid_pos`: A dictionary UID->genomic position (from, to, strand) 0-based
                  The right position is exclusive as python coords
     - `uid_names`: A dict UID->{category->name}
-    - `uid_tudata`: A dict UID->[TUs]
+    - `uid_tudata`: A dict UID->[IGTs]
     - `sRNAs list`: A list of sRNAs,
     - `other_RNAs_list`: a list of tRNAs, rRNAs etc
     - `rRNA_prod`: Name of rRNA product to return a list of rRNAs
@@ -258,6 +285,7 @@ def read_genes_data(
     other_RNAs_list = []
     rRNAs = []
     uid_tudata = {}
+    # import sys
     with open("%s/%s"%(ec_dir, gfile)) as ecin:
         ingene = None
         has_types = False
@@ -286,11 +314,11 @@ def read_genes_data(
             if line.startswith('COMPONENT-OF'):
                 tuname = line.strip().split()[-1]
                 if tuname.startswith('TU'):
-                    tu_data.append(tuname)
+                    tu_data.append(tuname.replace('TU', 'IGT'))
                 else:
                     chrom_name = tuname.rsplit('-',1)[0]
             if line.startswith('PRODUCT'):
-                if line.strip().split()[-1].split('-')[-1]==rRNA_prod:
+                if line.strip().split()[-1].split('-')[-1] in rRNA_prod:
                     rRNAs.append(ingene)
             if line.startswith('//'):
                 if coords[2] != '' and coords[1]>coords[0]:
@@ -325,35 +353,36 @@ def anti_strand(strand):
 
 def position_to_gene(
     uid_pos, uid_tu, tu_promoters, tu_terminators, tu_genes, sRNAs_list,
-    fsa_lens, rep_pos=None, utr_len=100):
+    fsa_lens, rep_pos=None, utr_len=100, lin_list=None):
     """
     For each postion (and strand) determines the gene in the position.
     If the real transcription is known use it. If between the TSS and the
-    gene start there is another gene, both are in the same TU,
-    this region is intergenic and will be marked as gene1_gene2_TU.
-    If the terminator is known use it to determine 3' end of the TU.
+    gene start there is another gene, both are in the same IGT,
+    this region is intergenic and will be marked as gene1_gene2_IGT.
+    If the terminator is known use it to determine 3' end of the IGT.
     If the gene is a sRNA use the end of the gene as 3' end
-    If there are multiple TUs associated with the gene take the longest one.
+    If there are multiple IGTs associated with the gene take the longest one.
     If the promoter/terminator is not known (not associated in the DB)
     use the default length to set 3'/5' UTRs.
     If the region is intergenic annotate is as gene1_gene2_IGR
     Arguments:
     - `uid_pos`: Dict Unique ID -> position
     - `uid_tu`: Unique ID -> Transcription units
-    - `tu_promoters`: TU ID -> promoter position (+1)
-    - `tu_terminators`: TU ID -> terminator positions (left, right)
-    - `tu_genes`: Dict TU ID -> [genes]
+    - `tu_promoters`: IGT ID -> promoter position (+1)
+    - `tu_terminators`: IGT ID -> terminator positions (left, right)
+    - `tu_genes`: Dict IGT ID -> [genes]
     - `sRNAs_list`: A list of sRNAs, ignore terminators and promoters
     - `fsa_lens`: Lengths of chromosomes
     - `rep_pos`: Positions of REP elements in the genome
     - `utr_len`: Default UTR length
+    - `lin_list`: List of linear chromosome/plasmid name
 
     Returns:
     - `pos_map`: A dictionary (position, strand) -> annotation
     """
     def first_in_TU(gname):
         """
-        If it's first in TU return the promoter name (TU). If not return None
+        If it's first in TU return the promoter name (IGT). If not return None
         Arguments:
         - `gname`: gene name
         """
@@ -390,7 +419,7 @@ def position_to_gene(
     
     def last_in_TU(gname):
         """
-        If it's last in TU return the terminator name (TU). If not return None
+        If it's last in TU return the terminator name (IGT). If not return None
         Arguments:
         - `gname`: gene name
         """
@@ -506,8 +535,11 @@ def position_to_gene(
                             (gname, 'EST5UTR', i-gpos[2])
 
         if est_3utr:
+            curr_chr_len = fsa_lens[gpos[0]]
             if gpos[3] == '+':
                 for i in range(gpos[2], gpos[2]+utr_len):
+
+                    i = i % curr_chr_len
                     if (i, gpos[3]) not in pos_map[gpos[0]]:
                         pos_map[gpos[0]][(i, gpos[3])] =\
                             (gname, 'EST3UTR', i-gpos[2])
@@ -528,6 +560,8 @@ def position_to_gene(
 
             else: # Minus strand
                 for i in range(gpos[1]-utr_len, gpos[1]):
+                    i = i % curr_chr_len
+                    
                     if (i, gpos[3]) not in pos_map[gpos[0]]:
                         pos_map[gpos[0]][(i, gpos[3])] =\
                             (gname, 'EST3UTR', gpos[1]-i)
@@ -608,6 +642,23 @@ def position_to_gene(
                                    len(pos_map[chrn][(j, '-')]) == 1])
         except ValueError:
             max_min_pos = None
+
+        # This handling is for cyclic genomes only
+        try:
+            min_plus_pos = min([j for j in range(chrlen) if\
+                                    (j, '+') in pos_map[chrn] and\
+                                    len(pos_map[chrn][(j, '+')]) == 1])
+        except ValueError:
+            min_plus_pos = None
+
+        try:
+            min_min_pos = min([j for j in range(chrlen) if\
+                                   (j, '-') in pos_map[chrn] and\
+                                   len(pos_map[chrn][(j, '-')]) == 1])
+        except ValueError:
+            min_min_pos = None
+        # end of cyclic code
+
         if max_plus_pos and max_plus_pos > max_min_pos:
             last_gene = pos_map[chrn][(max_plus_pos, '+')]
         elif max_min_pos:
@@ -637,24 +688,31 @@ def position_to_gene(
                     next_strand = '-'
                     break
                 next_pos += 1
+
             if next_strand == '+':
                 next_gene = pos_map[chrn][(next_pos, '+')]
             elif next_strand == '-':
                 next_gene = pos_map[chrn][(next_pos, '-')]
+            # This handling is for cyclic genomes only (otherwise use the commented else)
+            elif lin_list and chrn in lin_list:
+                next_gene = (chrn+'_End', )
             else:
-                next_gene = ('None', )
+                if min_plus_pos < min_min_pos:
+                    next_gene = pos_map[chrn][(min_plus_pos, '+')]
+                else:
+                    next_gene = pos_map[chrn][(min_min_pos, '-')]
 #        sys.stderr.write('%d\t%s\n'%(i, next_gene[0]))
             igr_plus_tag = 'IGR'
             igr_minus_tag = 'IGR'
-        # Test if the genes are in the same TU
+        # Test if the genes are in the same IGT
             for tu in uid_tu[last_gene[0]]:
                 if next_gene[0] in tu_genes[tu]:
                     if uid_pos[last_gene[0]][3] == '+':
-                        igr_plus_tag = 'TU'
-                        igr_minus_tag = 'TU_AS'
+                        igr_plus_tag = 'IGT'
+                        igr_minus_tag = 'IGT_AS'
                     else:
-                        igr_plus_tag = 'TU_AS'
-                        igr_minus_tag = 'TU'
+                        igr_plus_tag = 'IGT_AS'
+                        igr_minus_tag = 'IGT'
 
             while (((i, '+') not in pos_map[chrn]) or\
                        ((i, '-') not in pos_map[chrn])) and i < chrlen:
@@ -676,4 +734,6 @@ def position_to_gene(
                         pos_map[chrn][(i, anti_strand(rep_d[3]))] = tuple(
                             list(pos_map[chrn][(i, anti_strand(rep_d[3]))]) +\
                                 ['REP_%s_AS'%rep_name])
+
     return pos_map
+
