@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 
 """
-Generate a genes gff file using EcoCyc data given as flat-files
+Generate a genes gff file using BioCyc data given as flat-files
 """
 
 import sys
@@ -19,14 +19,14 @@ def process_command_line(argv):
 
     # initialize the parser object, replace the description
     parser = argparse.ArgumentParser(
-        description='Generate EcoCyc genes gff file')
+        description='Generate BioCyc genes gff file')
     parser.add_argument(
-        'ec_dir',
-        help='EcoCyc flat-files directory.')
+        'bc_dir',
+        help='BioCyc flat-files directory.')
     parser.add_argument(
-        '--EC_chrlist', default='COLI-K12,chr',
+        '--BC_chrlist', default='COLI-K12,chr',
         help='A comma separated dictionary of chromosome names from the '
-        'EcoCyc name to the bam name. See the names of chromosomes in bam file'
+        'BioCyc name to the bam name. See the names of chromosomes in bam file'
         ' using samtools view -H foo.bam.')
 
     settings = parser.parse_args(argv)
@@ -35,15 +35,15 @@ def process_command_line(argv):
 
 def main(argv=None):
     settings = process_command_line(argv)
-    if settings.EC_chrlist:
+    if settings.BC_chrlist:
         chr_dict = dict(zip(
-                settings.EC_chrlist.split(',')[0::2],
-                settings.EC_chrlist.split(',')[1::2]))
+                settings.BC_chrlist.split(',')[0::2],
+                settings.BC_chrlist.split(',')[1::2]))
     else:
         chr_dict = None
 
     ecocyc_parser.generate_gff_file(
-        sys.stdout, settings.ec_dir, chr_dict=chr_dict)
+        sys.stdout, settings.bc_dir, chr_dict=chr_dict)
     # application code here, like:
     # run(settings, args)
     return 0        # success
